package org.greenstone.gatherer.metadata;


import java.io.*;
import java.util.*;
import org.greenstone.gatherer.file.FileNode;
import org.greenstone.gatherer.util.Utility;
import org.greenstone.gatherer.util.XMLTools;


/** This class is a static class that manages the metadata.xml files */
public class MetadataXMLFileManager
{
    static private ArrayList metadata_xml_files = new ArrayList();


    static public void addMetadata(FileNode[] file_nodes, MetadataValue metadata_value)
    {
	for (int i = 0; i < file_nodes.length; i++) {
	    File current_file = file_nodes[i].getFile();
	    System.err.println("Adding metadata to " + current_file.getAbsolutePath());

	    // Find which metadata.xml file needs editing
	    boolean applicable_metadata_xml_file_found = false;
	    File current_file_directory = (current_file.isDirectory() ? current_file : current_file.getParentFile());
	    for (int j = 0; j < metadata_xml_files.size(); j++) {
		MetadataXMLFile metadata_xml_file = (MetadataXMLFile) metadata_xml_files.get(j);

		// This metadata.xml file is only potentially applicable if it is above or at the same level as the file
		if (current_file_directory.getAbsolutePath().startsWith(metadata_xml_file.getParentFile().getAbsolutePath())) {
		    applicable_metadata_xml_file_found = true;
		    metadata_xml_file.addMetadata(current_file, metadata_value);
		}
	    }

	    // If no applicable metadata.xml file exists, we have to create a new one
	    if (!applicable_metadata_xml_file_found) {
		// Create a new (empty) metadata.xml file in the file's directory...
		File new_metadata_xml_file_file = new File(current_file_directory, "metadata.xml");
		XMLTools.writeXMLFile(new_metadata_xml_file_file, Utility.parse("xml/metadata.xml", true));

		// ...load it...
		MetadataXMLFile new_metadata_xml_file = loadMetadataXMLFile(new_metadata_xml_file_file);

		// ...and add the metadata
		new_metadata_xml_file.addMetadata(current_file, metadata_value);
	    }
	}
    }


    static public void clearMetadataXMLFiles()
    {
	metadata_xml_files.clear();
    }


    static public ArrayList getMetadataAssignedToFile(File file)
    {
	// Build up a list of metadata values assigned to this file
	ArrayList metadata_values_all = new ArrayList();

	// Look at each loaded metadata.xml file to see if any assign metadata to this file
	File file_directory = (file.isDirectory() ? file : file.getParentFile());
	for (int i = 0; i < metadata_xml_files.size(); i++) {
	    MetadataXMLFile metadata_xml_file = (MetadataXMLFile) metadata_xml_files.get(i);

	    // This metadata.xml file is only potentially applicable if it is above or at the same level as the file
	    if (file_directory.getAbsolutePath().startsWith(metadata_xml_file.getParentFile().getAbsolutePath())) {
		ArrayList metadata_values = metadata_xml_file.getMetadataAssignedToFile(file);
		for (int j = 0; j < metadata_values.size(); j++) {
		    MetadataValue metadata_value = (MetadataValue) metadata_values.get(j);

		    // Overriding metadata: remove any values with this metadata element
		    if (metadata_value.isAccumulatingMetadata() == false) {
			for (int k = metadata_values_all.size() - 1; k >= 0; k--) {
			    if (((MetadataValue) metadata_values_all.get(k)).getMetadataElement().equals(metadata_value.getMetadataElement())) {
				metadata_values_all.remove(k);
			    }
			}
		    }

		    metadata_values_all.add(metadata_value);
		}
	    }
	}

	return metadata_values_all;
    }


    static public void loadMetadataXMLFiles(File directory)
    {
	// Make sure the directory (import) exists
	if (directory.exists() == false) {
	    return;
	}

	// Look recursively at each subfile of the directory for metadata.xml files
	File[] directory_files = directory.listFiles();
	for (int i = 0; i < directory_files.length; i++) {
	    File child_file = directory_files[i];
	    if (child_file.isDirectory()) {
		loadMetadataXMLFiles(child_file);
	    }
	    else if (child_file.getName().equals("metadata.xml")) {
		loadMetadataXMLFile(child_file);
	    }
	}
    }


    static private MetadataXMLFile loadMetadataXMLFile(File metadata_xml_file_file)
    {
	MetadataXMLFile metadata_xml_file = new MetadataXMLFile(metadata_xml_file_file.getAbsolutePath());
	metadata_xml_files.add(metadata_xml_file);
	return metadata_xml_file;
    }


    static public void removeMetadata(FileNode[] file_nodes, MetadataValue metadata_value)
    {
	for (int i = 0; i < file_nodes.length; i++) {
	    File current_file = file_nodes[i].getFile();
	    System.err.println("Removing metadata from " + current_file.getAbsolutePath());

	    // Find which metadata.xml file needs editing
	    File current_file_directory = (current_file.isDirectory() ? current_file : current_file.getParentFile());
	    for (int j = 0; j < metadata_xml_files.size(); j++) {
		MetadataXMLFile metadata_xml_file = (MetadataXMLFile) metadata_xml_files.get(j);

		// This metadata.xml file is only potentially applicable if it is above or at the same level as the file
		if (current_file_directory.getAbsolutePath().startsWith(metadata_xml_file.getParentFile().getAbsolutePath())) {
		    metadata_xml_file.removeMetadata(current_file, metadata_value);
		}
	    }
	}
    }
}
